/*
 * NOSA HEADER START
 *
 * The contents of this file are subject to the terms of the NASA Open 
 * Source Agreement (NOSA), Version 1.3 only (the "Agreement").  You may 
 * not use this file except in compliance with the Agreement.
 *
 * You can obtain a copy of the agreement at
 *   docs/NASA_Open_Source_Agreement_1.3.txt
 * or 
 *   http://cdaweb.gsfc.nasa.gov/WebServices/NASA_Open_Source_Agreement_1.3.txt.
 *
 * See the Agreement for the specific language governing permissions
 * and limitations under the Agreement.
 *
 * When distributing Covered Code, include this NOSA HEADER in each
 * file and include the Agreement file at 
 * docs/NASA_Open_Source_Agreement_1.3.txt.  If applicable, add the 
 * following below this NOSA HEADER, with the fields enclosed by 
 * brackets "[]" replaced with your own identifying information: 
 * Portions Copyright [yyyy] [name of copyright owner]
 *
 * NOSA HEADER END
 *
 * Copyright (c) 2011 United States Government as represented by 
 * the National Aeronautics and Space Administration. No copyright is 
 * claimed in the United States under Title 17, U.S.Code. All Other 
 * Rights Reserved.
 *
 * $Id: VariableNotes.java,v 1.3 2011/03/18 10:29:34 bharris Exp $
 */
package gsfc.spdf.istp.tools;


import gsfc.nssdc.cdf.CDFException;
import gsfc.nssdc.cdf.Variable;


/**
 * This class represents the "notes" obtained from a 
 * <a href="http://cdf.gsfc.nasa.gov/">Common Data Format (CDF)</a>
 * variable.  The "notes" are values which describe a variable and 
 * are displayed as "dataset notes" on 
 * <a href="http://cdaweb.gsfc.nasa.gov/">Coordinated Data Analysis Web</a>.
 *
 * @version $Revision: 1.3 $
 * @author B. Harris
 */
public class VariableNotes {

    /**
     * Value of the "CATDESC" attribute.
     */
    private String name = null;

    /**
     * Value of the "VAR_NOTES" attribute.
     */
    private String notes = null;


    /**
     * Constructs a VariableNotes object with the given values.  
     *
     * @param name variable name
     * @param notes variable notes
     */
    public VariableNotes(String name, String notes) {

        mergeValue(name, notes);
    }


    /**
     * Merges the given variable name and notes values with this object's
     * own values.  That is, unless the given values are null; blank; or,
     * in the case of a notes value, a '.'; they replace this object's
     * current values.
     *
     * @param name variable name
     * @param notes variable notes
     */
    public void mergeValue(String name, String notes) {

        if (name != null && !name.trim().equals("")) {

            setName(name);
        }

        if (notes != null && !notes.trim().equals("") && 
            !notes.equals(".")) {

            setNotes(notes);
        }
    }


    /**
     * Gets the "CATDESC" attribute value.
     *
     * @return the name value
     * @see #setName
     */
    public String getName() {

        return name;
    }


    /**
     * Sets the "CATDESC" attribute value.
     *
     * @param value new name value
     * @see #getName
     */
    public void setName(String value) {

        name = value;
    }


    /**
     * Gets the "VAR_NOTES" attribute value.
     *
     * @return the notes value
     * @see #setNotes
     */
    public String getNotes() {

        return notes;
    }


    /**
     * Sets the "VAR_NOTES" attribute value.
     *
     * @param value new notes value
     * @see #getNotes
     */
    public void setNotes(String value) {

        notes = value;
    }
}
